package fr.cnam.projet;

import java.util.*;
import java.text.*;

import fr.cnam.util.*;
import fr.cnam.ihm.*;
import fr.cnam.projet.*;

// Classe de definition d'une plate-forme de jeu
//   Cette plate-fome de jeu permet de jouer au jeu d'othelle
//    et gre les joueurs et les aprties jouees
//
// Pour ne pas saisir a chaque fois les joueurs et des parties jouees
//  la classe d'initialise en important des fichiers textes se trouvant
//  dans le repertoire data
//
public class PFJeu
{
  private Othello  othello;  // Le jeu d'othello (classe de modelisation du jeu)
  private IHMPFJeu ihm;      // Ihm de la plate-forme de jeu

  private ArrayList<Joueur> joueurs;  // La collection des joueurs
  private ArrayList<Partie> parties;  // La collection des parties jouees

  private String identJoueurCourant;  // L'identification du joueur courant
  private String identAdversaire;     // L'identification du joueur adverse

  // Constructeur de la plate-forme de jeu :
  //   initialisation par dfaut des attributs de la classe
  //
  public PFJeu()
  {
    othello = new Othello();
    joueurs = new ArrayList<Joueur>();
    parties = new ArrayList<Partie>();
    identJoueurCourant = "";
    identAdversaire = "";
  }

  // Initialisation de la plate-forme de jeu.
  // Cette methode est appelle par l'IHM
  //  une fois l'ihm initialise (pour connaitre la grille)
  //
  public void initialiser(IHMPFJeu ihm)
  {
    this.ihm = ihm;  // L'applicatif a aussi un lien sur l'IHM
    
    ihm.AR("Initialisation de la plate-forme de jeu :");
      
    // Lecture du fichier des joueurs inscrits
    //
    ihm.AR("Lecture du fichier des joueurs inscrits");
    String[] lignes = Terminal.lireFichierTexte("data/Joueurs.txt");
    if (lignes == null) 
      {
        ihm.AR("Impossible de lire le fichier data/Joueurs.txt");
        return;
      }
      
    for(String ligne:lignes)
      {
        String[] champs = ligne.split(";",2);
        String ident = champs[0];
        String mdp   = champs[1];
        Joueur joueur = new Joueur(ident,mdp);
        joueurs.add(joueur);
      }

    // Lecture du fichier des parties jouees
    //
    ihm.AR("Lecture du fichier des partie jouees");
    lignes = Terminal.lireFichierTexte("data/Parties.txt");
    if (lignes == null) 
      {
        ihm.AR("Impossible de lire le fichier data/Parties.txt");
        return;
      }
      
    for(String ligne:lignes)
      {
        String[] champs = ligne.split(";",7);
        int    numero           = Integer.parseInt(champs[0]);
        String identJoueur      = champs[1];
        String date             = champs[2];
        String nomJeu           = champs[3];
        int    terminee         = Integer.parseInt(champs[4]);
        int    score            = Integer.parseInt(champs[5]);
        int    scoreAdversaire  = Integer.parseInt(champs[6]);

        Partie partie = new Partie(numero,
                                   identJoueur,
                                   date,
                                   nomJeu,
                                   terminee,
                                   score,
                                   scoreAdversaire);
        parties.add(partie);
      }

    // Initialisation du jeu d'othello
    //
    othello.initialiser(this);
  }
    
  // Methode qui retourne sous la forme d'une chaine de caractere
  //  tous les joueurs
  //
  public void listerTousJoueurs()
  {
    ihm.AR("");
    for(Joueur j : joueurs)
      ihm.AR(j.toString());
  }
  
  // Methode qui retourne sous la forme d'une chaine de caractere
  //  toutes les parties
  //
  public void listerToutesParties()
  {
    ihm.AR("");
    for(Partie p : parties)
      ihm.AR(p.toString());
  }

  // Methode qui realise l'inscription d'un nouveau joueur
  //
  public void inscrire(String ident,String mdp)
  {
    if (ident.equals("")) 
      {
        ihm.AR("L'identifiant saisi est vide");
        return;
      }

    if (ident.indexOf(' ')!=-1)
      {
        ihm.AR("L'identifiant contient un caractre blanc");
        return;
      }

    for(Joueur j:joueurs)
      if (j.getIdent().equals(ident))
        {
          ihm.AR("L'identifiant existe deja.");
          return;
        }

    if (mdp.equals(""))
      {
        ihm.AR("Le mot de passe saisi est vide");
        return;
      }

    if (mdp.indexOf(' ')!=-1)
      {
        ihm.AR("Le mot de passe contient un caractre blanc");
        return;
      }

    if (mdp.length()>15)
      {
        ihm.AR("Le mot de passe doit etre inferieur ou egal a 15 caracteres");
        return;
      }

    // Ajout du nouveau joueur
    joueurs.add(new Joueur(ident,mdp));
    
    ihm.AR("Le nouveau joueur est inscrit");

  }


  // Methode qui realise l'identification d'un joueur
  //
  public void identifier(String ident,String mdp)
  {
    Joueur joueur = null;
    for(Joueur j:joueurs)
      if (j.getIdent().equals(ident)) 
        joueur=j;
    
    if (joueur==null)
      {
        ihm.AR("L'identifiant saisie n'existe pas");
        return;
      }

    if (! mdp.equals(joueur.getMdp()))
      {
        ihm.AR("Le mot de passe saisi n'est pas correct");
        return;
      }

    identJoueurCourant = ident;

    // On met a jour le nom du joueur courant dans l'ihm
    ihm.getForm().setValeurChamp("JOUEUR",ident);

    ihm.AR("Idenitification de "+ident+" ok.");
  }

  // Methode qui affiche les parties jouees du joueur
  //
  public void partiesJoueur()
  {
    if (identJoueurCourant.equals(""))
      {
        ihm.AR("Pas de joueur courant");
        return;
      }
    
    boolean ok=false;
    for(Partie p : parties)
      if (p.getIdentJoueur().equals(identJoueurCourant))
        {
          ihm.AR(p.toString());
          ok=true;
        }
    if (!ok) ihm.AR("Le joueur "+identJoueurCourant+" n'a pas de partie jouee");
  }

  // Choix de l'adversaire
  public void choisirAdversaire(String ident)
  {
    identAdversaire = ident;
    ihm.getForm().setValeurChamp("ADVERSAIRE",ident);
    ihm.AR("Choix de l'adversaire : "+ident);
  }

  // Demarrer une partie
  //
  public void demarrer()
  {
    if (identJoueurCourant.equals(""))
      {
        ihm.AR("Pas de joueur courant identifie");
        return;
      }

    if (identAdversaire.equals(""))
      {
        ihm.AR("Pas d'adversaire.");
        return;
      }

    othello.demarrer();
    
    ihm.AR("Partie demarree");
  }

  // Methode qui arrete une partie et memorise les scores
  //  de la partie
  //
  public void arreter()
  {
    // Il faut que la partie soit demarree
    if (! othello.getDemarree())
        {
          ihm.AR("La partie n'est pas demarree");
          return;
        }

    // La partie s'arrete
    //
    othello.setDemarree(false);

    // On memorise la partie jouee par les deux joueurs
    //
    int nbPionsJoueur1 = othello.nbPionsJoueur1();
    int nbPionsJoueur2 = othello.nbPionsJoueur2();

    SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
    StringBuffer sb = new StringBuffer();
    sdf.format(Calendar.getInstance().getTime(),sb,new FieldPosition(0));
    String date = sb.toString();

    int numPartie = numeroPartie();
    Partie p1 = new Partie(numPartie,
                           identJoueurCourant,
                           date,
                           "othello",
                           0,
                           nbPionsJoueur1,
                           nbPionsJoueur2);
    Partie p2 = new Partie(numPartie,
                           identAdversaire,
                           date,
                           "othello",
                           0,
                           nbPionsJoueur2,
                           nbPionsJoueur1);
    ihm.AR("Ajout des scores des parties jouees");
    parties.add(p1);
    parties.add(p2);
    
    ihm.AR("Arret de la partie");
  }

  // Methode qui retourne un nouveau numero de partie
  //  = le numero max +1
  //
  private int numeroPartie()
  {
    int max=0;
    for(Partie p:parties)
      if (p.getNumero()>max) max=p.getNumero();
    return max+1;
  }

  // Getteurs
  public Othello getOthello(){return othello;}
  public IHMPFJeu getIhm(){return ihm;}
  public ArrayList<Joueur> getJoueurs(){return joueurs;}
}